package Grid::GPM::Archive;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;
use Data::Dumper;
use Grid::GPT::Package;
use Grid::GPT::Filelist;
use Grid::GPM::FilelistFunctions;
use Cwd;
use File::Find;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
);
$VERSION = '0.01';

{
  my %environment;
  sub check_for_compress {
    my ($self) = shift;
    return $environment{'compress'} if defined $environment{'compress'};
    if (defined eval { require Compress::Zlib}) {
      $environment{'compress'} = 1;
    } else {
      $environment{'compress'} = 0;      
    }
    return $environment{'compress'};
  }
  sub get_target {

    if (! defined ($environment{'target'})) {
      my $gpath = $ENV{GPT_LOCATION};
      
      if (!defined($gpath))
        {
          $gpath = $ENV{GLOBUS_LOCATION};
          
        }
      
      if (!defined($gpath))
        {
          die "GPT_LOCATION or GLOBUS_LOCATION needs to be set before running this script"
        }
      $environment{'target'} = 
        `cat $gpath/etc/globus_package_management/globus_build.conf`;
      chomp($environment{'target'});
    }
    return $environment{'target'};
    
  }
  
    
}

if (defined eval { require Archive::Tar}) {
  require Archive::Tar;
} else {
  die "ERROR: Cannot find the module Archive::Tar
If it is installed in a non-standard location make sure that PERL5LIB points to the location.
You can get the module from www.cpan.org";
}

# Preloaded methods go here.
sub new {
  my ($class, %arg) = @_;
  my $me = {
            pkg => $arg{'pkg'},
	    verbose => $arg{'verbose'},
	    log => $arg{'log'},
	    static => $arg{'static'},
            filelist => $arg{'filelist'},
            srcdir => $arg{'srcdir'},
            installdir => $arg{'installdir'},
            corebuild => $arg{'corebuild'},
	   };

  $me->{'globusdir'} = $ENV{GLOBUS_LOCATION}; 
  $me->{'target'} = get_target();

   $me->{'name'} = $arg{'pkg'}->{'Name'};
   $me->{'version'} = $arg{'pkg'}->{'Version'}->label();

  bless $me, $class;

  return $me;
}

sub get_filelist {
  my ($me, %arg) = @_;
  my ($flavor, $pkgtype, $genfilelist) = 
    (
     $arg{'flavor'},
     $arg{'pkgtype'},
     $arg{'genfilelist'},
    );

  #  print "Files for flavor $flavor:\n";
  my $filelist;
  # filter out package types which are affected by building static.
  if (defined($me->{'static'})) {
    return undef  if $pkgtype eq 'rtl' or $pkgtype eq 'pgm';
  } else {
    return undef if $pkgtype eq 'pgm_static';
  }

  if (defined $genfilelist) {
    $filelist = $me->get_genfilelist($flavor, $pkgtype, $genfilelist);
    $me->install_metadata($flavor, $pkgtype, $filelist);
  } elsif (defined $me->{'filelist'}){
    $filelist = $me->get_xmlfilelist($flavor, $pkgtype);
    $me->install_metadata($flavor, $pkgtype, $filelist);
  } else {
    $filelist = $me->get_installed_filelist($flavor, $pkgtype);
    $filelist = $me->get_build_generated_filelist($flavor, $pkgtype) 
      if ! defined $filelist;
  }

  # check for empty packages
  return undef if ! defined $filelist;


  return undef if ! grep {! m!etc/globus_packages! } @$filelist;

  return $filelist;
}

sub get_genfilelist {
  my ($me, $flavor, $pkgtype, $genfilelist) = @_;

  # take care of generated filelists first
  return undef if $pkgtype ne "pgm" and 
    $pkgtype ne "pgm_static" and
    $pkgtype ne "rtl" and 
      $pkgtype ne "dev";

  return undef if $flavor eq 'noflavor';

  my $filessort = new Grid::GPT::Filelist(flavor =>$flavor,
                                           list => $genfilelist,
                                           noexpand => 1
                                           );
  if ($pkgtype eq "rtl") {
    my $result = [];
    $filessort->flavored_files();
    $filessort->extract_dynamic_libs();
    $filessort->add_package_metadata_files('rtl');
    my $list = $filessort->get_list();
    push @$result, @$list;
    $filessort->reset();
    $filessort->flavored_files();
    $filessort->extract_libtool_libs();
    $list = $filessort->get_list();
    push @$result, @$list;
    $filessort->reset();
    return $result;
  }
  
  if ($pkgtype eq 'dev') {
    my $result = [];
    $filessort->flavored_files();
    $filessort->extract_static_libs();
    my $list = $filessort->get_list();
    push @$result, @$list;
    $filessort->reset();
    $filessort->flavored_headers();
    $filessort->add_package_metadata_files('dev');
    $list = $filessort->get_list();
    push @$result, @$list;
    $filessort->reset();
    return $result;
  }
  
  if ($pkgtype eq 'pgm' or $pkgtype eq 'pgm_static') {
    $filessort->noflavor_files();
    if ($pkgtype eq 'pgm'){
    	$filessort->add_package_metadata_files('pgm');
    }else{
    	$filessort->add_package_metadata_files('pgm_static'); 
    }
    my $list = $filessort->get_list();
    return $list;
  }
}

sub get_xmlfilelist {
  my ($me, $flavor, $pkgtype) = @_;

  my @filelist;
  for my $f (@{$me->{'filelist'}}) {
    # skip entry if package type and flavor doesn't match.
    next if $f->{'Package_Type'} ne $pkgtype;
    next if $f->{'Flavored'} ne 'no' and $flavor eq 'noflavor';
    next if $f->{'Flavored'} eq 'no' and $flavor ne 'noflavor';
    
    my $dir = $f->{'Dir'};
    my @files = @{$f->{'Files'}};
    
    #modify filenames and directories for the flavor if necessary
    $dir =~ s!include/!include/$flavor/! if $f->{'Flavored'} eq 'headers';
    
    if ($f->{'Flavored'} eq 'libraries') {
      for my $f(@files) {
        $f =~ s!(lib[^.]+)\.!$ {1}_$flavor.!;
      }
    }
    push @filelist, @files;
  }
   return \@filelist; 
}

sub get_installed_filelist {
  my ($me, $flavor, $pkgtype) = @_;
  my $filelist_name = "$me->{'installdir'}/etc/globus_packages";
  $filelist_name .= "/$me->{'name'}/${flavor}_$pkgtype.filelist";

  return undef if ! -f $filelist_name;
      
  open (FILELIST, $filelist_name) or die "ERROR could not open $filelist_name\n";
  my @filelist = <FILELIST>;
  close FILELIST;
  for (@filelist) {
    s!^/+!!;
    chomp;
  }

  return \@filelist;
}

sub get_build_generated_filelist {
  my ($me, $flavor, $pkgtype) = @_;
  my $filelist_name = "$me->{'srcdir'}/filelist";

  return undef if ! -f $filelist_name;
      
  open (FILELIST, $filelist_name) or die "ERROR could not open $filelist_name\n";
  my @filelist = <FILELIST>;
  close FILELIST;
  for (@filelist) {
    s!^/+!!;
    chomp;
  }

  # assume metadata is not installed
  $me->install_metadata($flavor, $pkgtype, \@filelist);

  return \@filelist;
}

sub install_metadata {
  my ($me, $flavor, $pkgtype, $filelist) = @_;

  return if ! defined $filelist;

  my $metadatadir = "etc/globus_packages/$me->{'pkg'}->{'Name'}";
  Grid::GPM::FilelistFunctions::mkinstalldir("$me->{'installdir'}/$metadatadir");

  my ($filelistname, $metadataname) = 
    ("$metadatadir/$ {flavor}_$ {pkgtype}.filelist",
     "$metadatadir/pkg_data_$ {flavor}_$ {pkgtype}.gpt");

  my $bin_pkg = $me->{'pkg'}->convert_metadata($pkgtype, $flavor);
  $bin_pkg->output_metadata_file("$me->{'installdir'}/$metadataname");

  push @$filelist, ($filelistname, $metadataname);
  open (FILELIST, ">$me->{'installdir'}/$filelistname") 
    or die "ERROR: could not open $me->{'installdir'}/$filelistname\n";
  for (@$filelist) {
    print FILELIST "$_\n";
  }
  close FILELIST;
}

sub archive {
  my ($me, %arg) = @_;
  my ($notarball, $flavor, $genfilelist) = 
    ($arg{'notarball'},
     $arg{'flavor'},
     $arg{'filelist'},
    );
  my $startdir = cwd();
  chdir $me->{'installdir'};
  my $action = "CREATING";
  $action = "INSTALLING" if defined $notarball;
  my $got_some_files = 0;


  print "$action PACKAGES FOR $me->{'name'}-$flavor...";
  my $printed_skip;
  for my $t (@Grid::GPT::Definitions::package_types) {
    my $filelist = $me->get_filelist(flavor => $flavor, 
                                     pkgtype => $t, 
                                     genfilelist => $genfilelist);
    next if ! defined $filelist;
    next if defined $notarball;
    print "$t ";
    #    print "BEFORE:\n", $filelist, "\n";
    my $archive_ext = "tar.gz";
    $archive_ext = "tar" if ! check_for_compress();
    my $tarfile =
      "$startdir/$me->{'name'}-$me->{'version'}-$me->{'target'}-${flavor}-$t.$archive_ext";
    $tarfile =
      "$startdir/$me->{'name'}-$me->{'version'}-$me->{'static'}-$me->{'target'}-${flavor}-$t.$archive_ext"
        if $t eq 'pgm_static';
    #      print "$tarfile\n";
    my $tar = Archive::Tar->new();
    $tar->add_files(@$filelist);
    my @files = $tar->list_files();
    $got_some_files++ if @files;
    #   print  "AFTER:\n", @files, "\n";
    if (check_for_compress()) {
      $tar->write($tarfile, 9);
    } else {
      $tar->write($tarfile);        
    }
  }
  die "ERROR: No installed files found\n" if ! defined $notarball
    and ! $got_some_files 
      and $flavor ne 'noflavor'
        and ! defined $me->{'corebuild'};
  print "DONE\n";

  chdir $startdir;

}



# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPM:Archive - Perl extension for archiving globus binaries.

=head1 SYNOPSIS

  use Archive;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Archive was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut
