package Grid::GPM::Build;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;
use Data::Dumper;
use Grid::GPT::Package;
use Grid::GPM::Inform;
use Grid::GPM::BuildMacros;
use Cwd;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
);
$VERSION = '0.01';

# Preloaded methods go here.
sub new {
  my ($class, %arg) = @_;
  my $me = {
	    srcobj => $arg{'srcobj'},
	    flavors => $arg{'flavors'},
            build_instructions => $arg{'build_instructions'} || 
            [
             {command => "MAKE_GPTMACRO distclean"},
             {command => "CONFIGENV_GPTMACRO $arg{'srcobj'}->{'srcdir'}/configure CONFIGOPTS_GPTMACRO FLAVOR_CONFIGOPTS_GPTMACRO"},
             {command => "MAKE_GPTMACRO"},
             {command => "MAKE_GPTMACRO install"}
            ],
            ignore_errors => $arg{'ignore_errors'},
            globusdir => $arg{'globusdir'},
            name => $arg{'name'},
            force => $arg{'force'},
	   };

  $me->{'log'} = new Grid::GPM::Inform(verbose => $arg{'verbose'}, 
                                       log => $arg{'log'}
                                      );
  
  $ENV{'GPT_LOCATION'} = $ENV{'GLOBUS_LOCATION'} if ! defined $ENV{'GPT_LOCATION'};
  $ENV{'GLOBUS_LOCATION'} = $arg{'globusdir'};

  bless $me, $class;

  my $macros = $arg{'macros'};
  if (! defined($arg{'build_instructions'})) {

    $me->{'using default build instructions'}++;

  }
  # Add the install directory to the filelist
  if (defined ($me->{'srcobj'}->{'filelist'})) {
    $me->{'filelist'} = [];
    for my $f (@{$me->{'srcobj'}->{'filelist'}}) {
      chomp $f;
      push @{$me->{'filelist'}}, $f;
    }
  }
  $me->{'macros'} = new Grid::GPM::BuildMacros(srcobj => $arg{'srcobj'},
                                               globusdir => $arg{'globusdir'},
                                               flavors => $arg{'flavors'},
                                               macros => $macros,
                                               log => $me->{'log'},
                                               filelist => $me->{'filelist'},
                                               static => $arg{'static'},
                                               installed_flavors =>
                                               $arg{'installed_flavors'},
                                               flavor_choices =>
                                               $arg{'flavor_choices'},
                                              );

  return $me;
}
sub check_for_installed_metadata {
    my ($me, $flavor) = @_;
    my $metadir = "$me->{'globusdir'}/etc/globus_packages/$me->{'name'}";

    return 0 if ! -d $metadir;

    opendir(META, $metadir) || die "ERROR: Could not access $metadir\n";
    my $metaname = "pkg_data_$ {flavor}_";
    my @metafiles = grep {m!$metaname!} readdir META;

    closedir META;

    return @metafiles > 0;
}

sub build {
  my ($me, $flavor) = @_;
  my $returning = 1;

  if ($me->check_for_installed_metadata($flavor) and ! defined $me->{'force'}) {
    $me->{'log'}->inform("SKIPPING REBUILD of $flavor",1);
    
    #only execute the perl macros which fiddle with the filelist
    my @steps = grep { $_->{'command'} =~ m!RUN_FLAVOR_INSTALL_GPTMACRO! }
      @{$me->{'build_instructions'}};
        
    my $build_step = $me->{'macros'}->expand($steps[0],$flavor) if @steps;

    # Check for filelist.
    $me->{'filelist'} = $me->{'macros'}->{'flavored_filelist'};

    return $returning;
  }

  my $startdir = cwd();

  $me->{'log'}->inform("Changing to $me->{'srcobj'}->{'srcdir'}");
  chdir $me->{'srcobj'}->{'srcdir'};

  $me->{'log'}->inform("BUILDING FLAVOR $flavor",1);

  for my $bs (@{$me->{'build_instructions'}}) {

    my $build_step = $me->{'macros'}->expand($bs,$flavor);

    next if ! defined $build_step;

    # Perform the step
    my $result = $me->{'log'}->action($build_step);

    if ($result and $build_step !~ m!patch|clean!) { 
      if (defined $me->{'ignore_errors'}) {
        $me->{'log'}->inform("...SKIPPING $flavor",1);
        $returning = undef;
        last;
      } else {
      #results are bad print them out.
      die "ERROR: Build has failed\n";
      }
    }
  }

  chdir $startdir;
  $me->{'log'}->inform("Changing to $startdir");
  # Check for filelist.
  $me->{'filelist'} = $me->{'macros'}->{'flavored_filelist'};
  return $returning;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;

__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPM:Build - Perl extension for building globus binaries.

=head1 SYNOPSIS

  use Build;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Build was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut
