package Grid::GPM::FilelistFunctions;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;
use Data::Dumper;
use Grid::GPT::Package;
use Grid::GPM::Inform;
use Grid::GPM::ExpandSource;
use Cwd;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
);
$VERSION = '0.01';
{
  my $startdir = cwd();
  sub startdir { return $startdir; }
}
# Preloaded methods go here.
sub new {
  my ($class, %arg) = @_;
  my $me = {
            log => $arg{'log'},
            snap => {},
	   };

  die"ERROR: FilelistFunctions needs an installation directory\n" 
    if ! defined $arg{'installdir'};
  $me->{'installdir'} = Cwd::abs_path($arg{'installdir'}) || die "ERROR: can't access $arg{'installdir'}\n";
;

  bless $me, $class;

  return $me;
}


sub flavor_install {
  my ($me, $filelist, $flavor) = @_;
  my @flavored_filelist;
  my %sofiles;

  return $filelist if ! defined $filelist;

  for my $f (@$filelist) {
    my $name = $f;

    # Append the flavor to each library file.
    if ($f =~ m!lib/!)
      {
        $name =~ s!(lib[^.]+)\.!$ {1}_$flavor.!;
        next if -l "$me->{'installdir'}/$name";

        if ($name =~ m!\.so\.!) {
          my ($dir, $libname) = $name =~ m!(.+)/([^/]+)$!;

          if (! defined $sofiles{$dir}) {
            opendir(LIBS,"$me->{'installdir'}/$dir");
            $sofiles{$dir} = [ grep { m!\.so! } readdir LIBS ];
            closedir LIBS;
          }

          #get the actually lib version numbers.  \
          # This is because IRIX changes them.
          my $rootname = $libname;
          $rootname =~ s!\.so.+$!!;

          my @realnames = 
            grep {! -l "$me->{'installdir'}/$dir/$_" }
              grep { m!$rootname! } 
                @{$sofiles{$dir}};

          my ($version) = $realnames[0] =~ m!\.so(.+)!;


          $libname = "$ {rootname}.so$version";
          $name = "$dir/$libname";

          # create the softlinks
          my $startdir = cwd();
          chdir "$me->{'installdir'}/$dir";
          my $linkname = $libname;
          $linkname =~ s!(\.so.\d+).+$!$1!;
          my $result = `ln -f -s $libname $linkname` 
            if ! -l "$me->{'installdir'}/$linkname";
          push @flavored_filelist, "$dir/$linkname";
          $linkname =~ s!(\.so).+$!$1!;
          $result = `ln -f -s $libname $linkname` 
            if ! -l "$me->{'installdir'}/$linkname";
          push @flavored_filelist, "$dir/$linkname";
          chdir $startdir;
        }

        #Make sure a flavored library exists.

        if (! -f "$me->{'installdir'}/$name") {
          if (! -f "$me->{'installdir'}/$f") {
            $me->{'log'}->inform("WARNING $f not found",1);
            next;
          } else {
            $me->{'log'}->inform("WARNING $f is not flavored",1);
          }

          if (-l "$me->{'installdir'}/$f") {
            my $result = `rm "$me->{'installdir'}/$f"`;
          }
        }
      }
 
    # Assume all headers are flavored
   if ($f =~ m!/?include/!) {
      my $dir = $f;
      $dir =~ s!/[^/]+\.h!!;
      $dir =~ s!^/?include!include/$flavor!;
      mkinstalldir("$me->{'installdir'}/$dir");
      $name =~ s!^/?include/!include/$flavor/!;
      my $result = `mv $me->{'installdir'}/$f $me->{'installdir'}/$name` 
        if -f "$me->{'installdir'}/$f";
    }

    if (! -f "$me->{'installdir'}/$name" and
        $f !~ m!/lib! and
        ! -l "$me->{'installdir'}/$name") {
      $me->{'log'}->inform("WARNING $name not found",1);
      next;
    }



    $name =~ s!/+!/!g;
    push @flavored_filelist, $name;
  }
  return \@flavored_filelist;
}

sub abspath {
  my ($file) = @_;
  my $home = $ENV{'HOME'};
  $file =~ s!~!$home!;
  my $startd = startdir();
  $file =~ s!^\./!$startd/!;
  $file = "$startd/$file" if $file !~ m!^\s*/!;
  return $file;
}

sub mkinstalldir {
  my $dir = abspath(shift);
  my @dirlist = split m!/!, $dir;
  my $subdir= "";
  for my $d (@dirlist) {
    $subdir .= "/$d";
    if (! -d $subdir) {
      my $result = `mkdir $subdir`;
    }
  }
}

# Autolod methods go after =cut, and are processed by the autosplit program.

1;

__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPM:FilelistFunctions - Perl extension for building globus binaries.

=head1 SYNOPSIS

  use Build;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Build was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut
