package Grid::GPM::Inform;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;
use Data::Dumper;
use Grid::GPT::Package;
use Grid::GPM::FilelistFunctions;
use Cwd;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
);
$VERSION = '0.01';

# Preloaded methods go here.
sub new {
  my ($class, %arg) = @_;
  my $me = {
	    verbose => $arg{'verbose'},
	    log => $arg{'log'},
	   };

  
  $me->{'log'} = Grid::GPM::FilelistFunctions::abspath($me->{'log'}) 
    if defined $me->{'log'};

  $me->{'verbose'} = 0 if ! defined  $me->{'verbose'};

  $|++;
  bless $me, $class;

  return $me;
}

sub inform {
  my ($me, $content, $override, $fh) = @_;
  $override = 0 if ! defined $override;

  if (defined $me->{'log'}) {
    open (LOG, ">>$me->{'log'}") || die "ERROR: Could not open $me->{'log'}\n";
    print LOG "$content\n";
    close LOG;
  } else {
    print "$content\n" if $me->{'verbose'} or $override;
    print $fh "$content\n" if defined $fh and 
      ($me->{'verbose'} or $override);
  }
}

sub action {
  my ($me, $command) = @_;

  # Log the step
  $me->inform($command);
  
  my $result;
  # Perform the step
  if (defined $me->{'log'}) {
    open (STDERR, ">gpt_errorlog");
    my $output = `$command`;
    $result = $?;
    close STDERR;
    open (ERRLOG, "gpt_errorlog");
#    $result=0;
    
    for my $l (<ERRLOG>) {
      $output .= $l;
#      $result= 1 if $l =~ m![Ee]rror! and $l !~ m!ignored!;
    }
    close ERRLOG;
    $me->inform($output);
  } else {
     my $output = `$command 2>&1`;
     $me->inform($output);
     if ($? and ! $me->{'verbose'}) {
       $me->inform($command,1);
       $me->inform($output, 1);
     }
     $result = $?;
  }

  return $result;

}


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPM:Inform - Perl extension for informing globus binaries.

=head1 SYNOPSIS

  use Inform;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Inform was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut
